# 将源码和依赖包打包至.zip
import os
import shutil


# 压缩文件

from pymvn.parsepybuild import get_root_path, get_site_packages_path, get_main_func_path


def zipdir(dirpath: str, filename: str, format: str):
    shutil.make_archive(filename, format=format, root_dir=dirpath)


# 将根目录下的所有文件及文件夹及site-packages文件夹打包在一个文件夹里
# project_name
#     src
#     site_packages
#     other_dir
def package():
    root_path = os.getcwd()
    site_packages_path = f"{root_path}/{get_site_packages_path()}"
    prj_name = os.path.split(root_path)[1]
    objs = os.listdir(root_path)
    root_path_copy = f"{root_path}/{prj_name}"
    if os.path.exists(f"{root_path}/{prj_name}"):
        os.remove(f"{root_path}/{prj_name}")
    os.mkdir(f"{root_path}/{prj_name}")
    for obj in objs:
        if obj == prj_name:
            continue
        final_path = f"{root_path}/{obj}"
        final_path_copy = f"{root_path_copy}/{obj}"
        if os.path.isdir(final_path):
            if obj != "venv":
                shutil.copytree(final_path, final_path_copy)
            else:
                # 将site_packages复制到prj_name下
                final_path = site_packages_path
                final_path_copy = f"{final_path_copy}/site-packages"
                shutil.copytree(final_path, final_path_copy)
        elif os.path.isfile(final_path):
            shutil.copyfile(final_path, final_path_copy)
    generate_mainpy(prj_name)
    zipdir(f"{root_path}/{prj_name}", prj_name, "zip")
    # 删除prj_name
    # os.remove(f"{root_path}/{prj_name}")


def generate_mainpy(prj_name: str):
    module_path = os.path.split(__file__)[0]
    # 解析__main_.py-tpl
    mainpy_path = f"{os.getcwd()}/{prj_name}/__main__.py"
    if os.path.exists(mainpy_path):
        os.remove(mainpy_path)
    with open(mainpy_path, mode="w", encoding="utf-8") as f:
        pass
    with open(mainpy_path, mode="w", encoding="utf-8") as fw:
        with open(f"{module_path}/__main__tpl.py", mode="r", encoding="utf-8") as f:
            for line in f:
                if not line or '"""' in line:
                    continue
                if "{{site_packages_path}}" in line:
                    line = line.replace("{{site_packages_path}}", "venv/site-packages")
                if "{{main_func_path}}" in line:
                    line = line.replace("{{main_func_path}}", get_main_func_path())
                fw.write(line)




# 初始化项目
import os

from loguru import logger

from pymvn.parsepybuild import get_root_path


def init():
    root_path = os.getcwd()
    pybuild_path = f"{root_path}/.pymvn"
    if not os.path.exists(pybuild_path):
        with open(pybuild_path, mode="w", encoding="utf-8") as f:
            f.write("main_func = \n")
            f.write("site_packages_path = \n")



# 解析.pybuild文件
# ：主函数
import os

_root_path = None
_main_func = None
_site_packages_path = None


def get_root_path():
    global _root_path
    if not _root_path:
        _root_path = os.path.split(os.path.split(__file__)[0])[0]
    return _root_path


def get_main_func_path():
    global _main_func, _site_packages_path
    if not _main_func or not _site_packages_path:
        parsepybuild()
    return _main_func


def get_site_packages_path():
    global _main_func, _site_packages_path
    if not _main_func or not _site_packages_path:
        parsepybuild()
    return _site_packages_path


# 解析参数main_func,site_packages_path
def parsepybuild():
    global _main_func, _site_packages_path
    root_path = get_root_path()
    pybuild_file = f"{os.getcwd()}/.pymvn"
    if not os.path.exists(pybuild_file):
        with open(pybuild_file, mode="w", encoding="utf-8") as f:
            pass
    if not os.path.exists(pybuild_file):
        raise Exception("the .pymvn file not in the project root path")
    with open(pybuild_file, mode="r", encoding="utf-8") as f:
        for line in f:
            if not line:
                continue
            else:
                k, v = line.strip().split("=")
                k = k.strip()
                v = v.strip()
                if k not in {"main_func", "site_packages_path"}:
                    raise KeyError(f".pybuilld key error, don't known {k}")
                if not v:
                    raise ValueError(f"nothing for {k}")
                if k == "main_func":
                    _main_func = v
                if k == "site_packages_path":
                    _site_packages_path = v






"""
# __main__.py模板
import sys
import importlib
import os


if __name__ == "__main__":
    # 设置site-packages
    prj_path = os.path.split(os.path.abspath(__file__))[0]
    site_packages_path = f"{prj_path}/{{site_packages_path}}"
    sys.path.append(site_packages_path)
    # main_func
    main_func_path = "{{main_func_path}}"
    items = main_func_path.split(".")
    func = items[-1]
    module = items[-2]
    path_ = ".".join(items[: -2])
    sys.path.append(path_)
    # 导入module，执行func
    if not path_:
        module_obj = importlib.import_module(module)
    else:
        module_obj = importlib.import_module(f"{path_}.{module}")
    func_obj = getattr(module_obj, func)
    func_obj()
"""



from setuptools import setup, find_packages


setup(
    name='pymvn',
    version="0.0.4",
    description=(
        """package python src and site-packages into *.zip, 
           then use `python *.zip` to run your program in any
           computer which has the correct python interpreter!,
           for current version,your project must use virtualenv"""
    ),
    author="zhangming",
    author_email="1570977353@qq.com",
    maintainer="zhangming",
    maintainer_email="1570977353@qq.com",
    license='BSD License',
    packages=find_packages(),
    include_package_data=True,
    package_data={"pymvn": [
        "*.py-tpl"
    ]},
    platforms=["all"],
    url="https://github.com/Mng12345/pymvn",
    classifiers=[
        'Development Status :: 4 - Beta',
        'Operating System :: OS Independent',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Programming Language :: Python',
        'Programming Language :: Python :: Implementation',
        'Programming Language :: Python :: 3.7',
        'Topic :: Software Development :: Libraries'
    ]
)


import sys

from pymvn.init import *
from pymvn.integrate import *


if __name__ == "__main__":
    if len(sys.argv) == 1:
        package()
    elif sys.argv[1] == "init":
        init()
        sys.exit(0)

